@@ -18,7 +18,7 @@
 #include "../lib/CHeroHandler.h"
 #include "../lib/CTownHandler.h"
 #include "../lib/CBuildingHandler.h"
-#include "../lib/CSpellHandler.h"
+#include "../lib/spells/CSpellHandler.h"
 #include "../lib/Connection.h"
 #ifndef VCMI_ANDROID
 #include "../lib/Interprocess.h"
@@ -150,7 +150,11 @@ void CClient::waitForMoveAndSend(PlayerColor color)
         logNetwork->debugStream() << "Wait for move thread was interrupted and no action will be send. Was a battle ended by spell?";
 		return;
 	}
-	HANDLE_EXCEPTION
+	catch(...)
+	{
+		handleException();
+		return;
+	}
     logNetwork->errorStream() << "We should not be here!";
 }
 
@@ -331,7 +335,7 @@ void CClient::loadGame(const std::string & fname, const bool server, const std::
          std::cout << x << std::endl;
     std::cout << "ENDCLIENTPLAYERS\n";
 
-    serialize(*loader,0,clientPlayers);
+    serialize(loader->serializer,0,clientPlayers);
     *serv << ui32(clientPlayers.size());
     for(auto & elem : clientPlayers)
         *serv << ui8(elem.getNum());
@@ -493,11 +497,10 @@ void CClient::newGame( CConnection *con, StartInfo *si )
 // 	}
 }
 
-template <typename Handler>
-void CClient::serialize( Handler &h, const int version )
+void CClient::serialize(COSer & h, const int version)
 {
-	h & hotSeat;
-	if(h.saving)
+	assert(h.saving);
+	h & hotSeat;	
 	{
 		ui8 players = playerint.size();
 		h & players;
@@ -507,11 +510,15 @@ void CClient::serialize( Handler &h, const int version )
 			LOG_TRACE_PARAMS(logGlobal, "Saving player %s interface", i->first);
 			assert(i->first == i->second->playerID);
 			h & i->first & i->second->dllName & i->second->human;
-			i->second->saveGame(dynamic_cast<COSer<CSaveFile>&>(h), version); 
-			//evil cast that i still like better than sfinae-magic. If I had a "static if"...
+			i->second->saveGame(h, version); 			
 		}
 	}
-	else
+}
+
+void CClient::serialize(CISer & h, const int version)
+{
+	assert(!h.saving);
+	h & hotSeat;
 	{
 		ui8 players = 0; //fix for uninitialized warning
 		h & players;
@@ -551,7 +558,7 @@ void CClient::serialize( Handler &h, const int version )
 			nInt->playerID = pid;
 
 			installNewPlayerInterface(nInt, pid);
-			nInt->loadGame(dynamic_cast<CISer<CLoadFile>&>(h), version); //another evil cast, check above
+			nInt->loadGame(h, version); //another evil cast, check above
 		}
 
 		if(!vstd::contains(battleints, PlayerColor::NEUTRAL))
@@ -559,11 +566,10 @@ void CClient::serialize( Handler &h, const int version )
 	}
 }
 
-template <typename Handler>
-void CClient::serialize( Handler &h, const int version, const std::set<PlayerColor>& playerIDs)
+void CClient::serialize(COSer & h, const int version, const std::set<PlayerColor> & playerIDs)
 {
+	assert(h.saving);
 	h & hotSeat;
-	if(h.saving)
 	{
 		ui8 players = playerint.size();
 		h & players;
@@ -573,11 +579,15 @@ void CClient::serialize( Handler &h, const int version, const std::set<PlayerCol
 			LOG_TRACE_PARAMS(logGlobal, "Saving player %s interface", i->first);
 			assert(i->first == i->second->playerID);
 			h & i->first & i->second->dllName & i->second->human;
-			i->second->saveGame(dynamic_cast<COSer<CSaveFile>&>(h), version); 
-			//evil cast that i still like better than sfinae-magic. If I had a "static if"...
+			i->second->saveGame(h, version); 			
 		}
 	}
-	else
+}
+
+void CClient::serialize(CISer & h, const int version, const std::set<PlayerColor> & playerIDs)
+{
+	assert(!h.saving);
+	h & hotSeat;
 	{
 		ui8 players = 0; //fix for uninitialized warning
 		h & players;
@@ -620,7 +630,7 @@ void CClient::serialize( Handler &h, const int version, const std::set<PlayerCol
             if(playerIDs.count(pid))
                  installNewPlayerInterface(nInt, pid);
 
-            nInt->loadGame(dynamic_cast<CISer<CLoadFile>&>(h), version); //another evil cast, check above            
+            nInt->loadGame(h, version);       
 		}
 
 		if(playerIDs.count(PlayerColor::NEUTRAL))
@@ -781,7 +791,11 @@ void CClient::commenceTacticPhaseForInt(shared_ptr<CBattleGameInterface> battleI
 			MakeAction ma(BattleAction::makeEndOFTacticPhase(gs->curB->playerToSide(battleInt->playerID)));
 			sendRequest(&ma, battleInt->playerID);
 		}
-	} HANDLE_EXCEPTION
+	}
+	catch(...)
+	{
+		handleException();
+	}	
 }
 
 void CClient::invalidatePaths()
@@ -901,8 +915,6 @@ std::string CClient::aiNameForPlayer(const PlayerSettings &ps, bool battleAI)
 	return goodAI;
 }
 
-template void CClient::serialize( CISer<CLoadFile> &h, const int version );
-template void CClient::serialize( COSer<CSaveFile> &h, const int version );
 
 void CServerHandler::startServer()
 {
@@ -960,7 +972,13 @@ CServerHandler::CServerHandler(bool runServer /*= false*/)
 	try
 	{
 		shared = new SharedMem();
-    } HANDLE_EXCEPTIONC(logNetwork->errorStream() << "Cannot open interprocess memory: ";)
+    }
+    catch(...)
+    {
+    	logNetwork->error("Cannot open interprocess memory.");
+    	handleException();
+    	throw;
+    }
 #endif
 }
 
