@@ -70,7 +70,10 @@ CModListView::CModListView(QWidget *parent) :
 	ui->progressWidget->setVisible(false);
 	dlManager = nullptr;
 	disableModInfo();
-	loadRepositories();
+	if (settings["launcher"]["autoCheckRepositories"].Bool())
+	{
+		loadRepositories();
+	}
 }
 
 void CModListView::loadRepositories()
@@ -99,7 +102,10 @@ void CModListView::showEvent(QShowEvent * event)
 	if (repositoriesChanged)
 	{
 		repositoriesChanged = false;
-		loadRepositories();
+		if (settings["launcher"]["autoCheckRepositories"].Bool())
+		{
+			loadRepositories();
+		}
 	}
 }
 
@@ -620,6 +626,11 @@ void CModListView::installMods(QStringList archives)
 	checkManagerErrors();
 }
 
+void CModListView::on_refreshButton_clicked()
+{
+	loadRepositories();
+}
+
 void CModListView::on_pushButton_clicked()
 {
 	delete dlManager;
